local LMP = LibMediaProvider

local EsoKR = {}
EsoKR.name = "EsoKR"
EsoKR.Flags = { "en", "kr" }
EsoKR.Version = "v0.17.0.3"
EsoKR.versionInt = 62
EsoKR.API = 100035
EsoKR.chat = { changed = true, privCursorPos = 0, editing = false }

-- EsoKR.firstInit = true
EsoKR.Defaults = {
    FirstInit = true,
    versionInt = 1
}
EsoKR.Settings = EsoKR.Defaults

local Font_Light = "EsoKR/fonts/univers57.otf"
local Font_Medium = "EsoKR/fonts/univers57.otf"
local Font_Bold = "EsoKR/fonts/univers67.otf"

local THIN          = 'soft-shadow-thin'
local THICK         = 'soft-shadow-thick'
local SHADOW        = 'shadow'
local NONE          = 'none'

EsoKR.FONTSTYLE_VALUES =
{
  FONT_STYLE_NORMAL,
  FONT_STYLE_OUTLINE,
  FONT_STYLE_OUTLINE_THICK,
  FONT_STYLE_SHADOW,
  FONT_STYLE_SOFT_SHADOW_THICK,
  FONT_STYLE_SOFT_SHADOW_THIN,
}

local panelData = {
    type = "panel",
    name = "微攻略汉化插件",
    displayName = "微攻略人工版汉化插件",
    author = "ESO微攻略汉化组",
    version = EsoKR.Version,
    registerForRefresh = true,
    registerForDefaults = true,
	website = "https://vstab.com/local",
}

local optionsTable = {
	[1] = {
		type = "header",
		name = "选项",
		width = "full",	--or "half" (optional)
	},
	
	[2] = {
		type = "header",
		name = "语言选择",
		width = "full",	--or "half" (optional)
	},
	
	[3] = {
		type = "checkbox",
		name = "自动保存上次使用语言",
		tooltip = "启用此选项后，游戏将使用上次选择的语言。否则，每次启动时都是英语。",
		getFunc = function()
			if GetCVar("IgnorePatcherLanguageSetting") == "1" then
				return true
			else
				return false
			end
		return GetCVar("IgnorePatcherLanguageSetting") end,
		setFunc = function(value)
			if value == true then
				SetCVar("IgnorePatcherLanguageSetting", "1")
			else
				SetCVar("IgnorePatcherLanguageSetting", "0")
			end
		end,
	},
	[4] = {
		type = "header",
		name = "社群",
		width = "full",	--or "half" (optional)
	},
	[5] = {
		type = "button",
		name = "美服公会",
		tooltip = "加入美服合作公会 Pigeon Legion鸽子军团",
		func = function()
		if GetWorldName() == "NA Megaserver" then
			if ZO_ValidatePlayerGuildId(665471) then
				EsoKR:ShowMsgBox("提示", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al你已经加入了Pigeon Legion鸽子军团。", 2)
			end
			ZO_LinkHandler_OnLinkClicked("|H1:guild:665471|hPigeon Legion|h", 1, nil)
			else
				EsoKR:ShowMsgBox("提示", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al这边检测到亲亲不是美服呢，如果是欧服的话请点击 欧服公会 的按钮！", 2)
		end end,
		width = "half",	--or "half" (optional)
	},
	[6] = {
		type = "button",
		name = "欧服公会",
		tooltip = "加入欧服合作公会 Splendid Achievers辉煌成就者",
		func = function() if GetWorldName() == "EU Megaserver" then
			if ZO_ValidatePlayerGuildId(641060) then
				EsoKR:ShowMsgBox("提示", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al你已经加入了Splendid Achievers辉煌成就者。", 2)
			end
			ZO_LinkHandler_OnLinkClicked("|H1:guild:641060|hSplendid Achievers|h", 1, nil)
			else
				EsoKR:ShowMsgBox("提示", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al这边检测到亲亲不是欧服呢，如果是美服的话请点击 美服公会 的按钮！", 2)
		end end,
		width = "half",	--or "half" (optional)
	},
	[7] = {
		type = "button",
		name = "加入Steam组",
		tooltip = "加入Steam ESOCN组，可第一时间获得汉化更新消息或是反映汉化问题！",
		func = function() 
			EsoKR:ShowMsgBox("提示", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al加入微攻略ESO CN Steam社区组，将会在默认浏览器内打开地址。中国大陆地区玩家可能无法正常访问Steam社区页面。", 4)
		end,
		width = "half",	--or "half" (optional)
	},
	[8] = {
		type = "button",
		name = "加入汉化组",
		tooltip = "汉化组一直在招新，如果你愿意为汉化插件贡献一份力量且满足要求的话，欢迎加入！",
		func = function() 
			EsoKR:ShowMsgBox("提示", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al将会在默认浏览器内打开申请地址，确认窗口中通过即可。", 5)
		end,
		width = "half",	--or "half" (optional)
	},
	[9] = {
		type = "header",
		name = "捐助打赏",
		width = "full",	--or "half" (optional)
	},
	
	[10] = {
		type = "description",
		text ="\n\n\n\n\n\n|ac|t512:256:EsoKR/Textures/Donate.dds|t\n\n\n\n\n\n|al汉化插件需要你的捐助支持！\n\n自2019年接手汉化插件之后，我独自为汉化组开发了翻译工具，从最初的将就用到现在的联网同步翻译，由于现在翻译工具的联网功能，服务器的费用是较大的一项开支。所以才公开求打赏捐助。学生请不要大额捐助。\n\nBevis熊熊",
		width = "full",	--or "half" (optional)
	},
}

function EsoKR:utfstrlen(input)
    local length = #input
    local left = length
    local cnt = 0
    local arr={0,0xc0,0xe0,0xf0,0xf8,0xfc}
    while left ~= 0 do
        local tmp=string.byte(input,-left)
        local i=#arr
        while arr[i] do
            if tmp>=arr[i] then 
              left=left-i
              break
            end
            i=i-1
        end
        cnt=cnt+1
    end
    return cnt
end

function EsoKR:con2CNKR(text, encode)
    local temp = ""
    local scanleft = 0
    local result = ""
    local num = 0
    local hashan = false;

    if(text == nil) then text = "" end
    for i in string.gmatch(text, ".") do
        --[[if(num >= 39) and hashan then
            temp = ""
            scanleft = 0
            result = utf8sub(result, 1, 40)
        else]]--
        local r = ""
        charPos = string.byte(i)
        hex = string.format('%02X', charPos)
        if scanleft > 0 then
            temp = temp .. hex
            scanleft = scanleft - 1
            if scanleft == 0 then
                temp = tonumber(temp, 16)
                if temp >= 0xE18480 and temp <= 0xE187BF then temp = temp + 0x43400
                elseif temp > 0xE384B0 and temp <= 0xE384BF then temp = temp + 0x237D0
                elseif temp > 0xE38580 and temp <= 0xE3868F then temp = temp + 0x23710
                elseif temp >= 0xEAB080 and temp <= 0xED9EAC then
                    if temp >= 0xEAB880 and temp <= 0xEABFBF then temp = temp - 0x33800
                        elseif temp >= 0xEBB880 and temp <= 0xEBBFBF then temp = temp - 0x33800
                        elseif temp >= 0xECB880 and temp <= 0xECBFBF then temp = temp - 0x33800
                        else temp = temp - 0x3F800
                    end
                elseif not encode and temp >= 0xE6B880 and temp <= 0xE9A6A3 then temp = temp + 0x3F800
                end
                temp = string.format('%02X', temp)
                r = (temp:gsub('..', function (cc) return string.char(tonumber(cc, 16)) end))
                temp = ""
                hashan = true
                num = num + 1
            end
        else
            if charPos > 0xE0 and charPos <= 0xEF then
                -- 3 bytes
                scanleft = 2
                temp = hex
            else
                r = i
                num = num + 1
            end
        end
        result = result .. r
        --end
    end
    return result
end

function EsoKR:Convert(edit)
    if self.chat.editing then return end
    local cursorPos = edit:GetCursorPosition()
    if cursorPos ~= self.chat.privCursorPos and cursorPos ~= 0 then
        self.chat.editing = true
        local text = self:con2CNKR(edit:GetText(), true)
        edit:SetText(text)
        if(cursorPos < EsoKR:utfstrlen(text)) then edit:SetCursorPosition(cursorPos) end
        self.chat.editing = false
    end
    self.chat.privCursorPos = cursorPos
end

-- 返回当前游戏语言，中文或英文
function EsoKR.GetLanguage()
    local lang = GetCVar("language.2")
    if (lang == "kr") then return lang end
    return "en"
end

-- 改变游戏语言
function EsoKR.ChangeLang(lang)
    zo_callLater(function()
        SetCVar("language.2", lang)
       -- EsoKR.savedVars.lang = lang
        ReloadUI()
    end, 500)
end

-- 是否是全 ascii 字符串，用来判断是否汉化了
function EsoKR.IsAscii(s)
    for i = 1, #s do
       if string.byte(string.sub(s, i, i)) >= 128 then
            return true
        end
    end
    return false
end

function EsoKR:SCTFix()
	if EsoKR:GetLanguage() == "kr" then
		SetSCTKeyboardFont(Font_Medium .. "|" .. 32 .. "|",FONT_STYLE_OUTLINE_THICK)
		SetSCTGamepadFont(Font_Medium .. "|" .. 42 .. "|",FONT_STYLE_SOFT_SHADOW_THICK)
		SetNameplateKeyboardFont(Font_Medium .. "|" .. 22 .. "|",FONT_STYLE_OUTLINE_THICK)
    SetNameplateGamepadFont(Font_Medium .. "|" .. 22 .. "|",FONT_STYLE_OUTLINE_THICK)
	end
end

function EsoKR:StartupMessage()
	if EsoKR.API ~= GetAPIVersion() then
		EsoKR:ShowMsgBox("汉化插件需要更新", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al汉化插件看起来已经过时，使用旧版的汉化插件可能会引起文本缺失、文本无法对应等问题。\n\n请到微攻略网站查看并下载最新版本的汉化插件。\n\nhttps://vstab.com/local \n\n你也可以直接点击下边的 访问网站 按钮，然后确认提示，会在你电脑的默认浏览器内自动打开汉化插件的地址。", 3)
	end

	if EsoKR.Settings.versionInt ~= EsoKR.versionInt then
	    EsoKR.Settings.versionInt = EsoKR.versionInt
		EsoKR:ShowMsgBox("已更新至v0.17.0.3", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al汉化插件本次更新内容如下：\n\n新增的CP星座翻译补全。\n技能与装备部分更新翻译。\n旅伴巴斯蒂安的解锁任务内容补全。\n耕战系统的相关翻译。（包括日常周常）\n天际西部还在逐渐翻译。\nPVP内容开始有小伙伴查找补缺。\n绿荫、迪沙安、盗贼公会的遗漏开始有小伙伴查找补缺。\n书籍因本身各种长文本，翻译较慢，但还在补充中。\n插件本体修复手柄模式的口口口问题。", 2)
		--EsoKR.Settings.version = EsoKR.versionInt
	end
	
	EVENT_MANAGER:UnregisterForEvent("EsoKR_StartupMessage", EVENT_PLAYER_ACTIVATED)
end

-- 重新加载 UI
function EsoKR.RefreshUI()
    local flagControl
    local count = 0
    local flagTexture
    for _, flagCode in pairs(EsoKR.Flags) do
        flagTexture = "EsoKR/flags/"..flagCode..".dds"
        flagControl = GetControl("EsoKR_FlagControl_"..tostring(flagCode))
        if flagControl == nil then
            flagControl = CreateControlFromVirtual("EsoKR_FlagControl_", EsoKRUI, "EsoKR_FlagControl", tostring(flagCode))
            GetControl("EsoKR_FlagControl_"..flagCode.."Texture"):SetTexture(flagTexture)
            if EsoKR.GetLanguage() ~= flagCode then
                flagControl:SetAlpha(0.3)
                if flagControl:GetHandler("OnMouseDown") == nil then
                    flagControl:SetHandler("OnMouseDown", function() EsoKR.ChangeLang(flagCode) end)
                end
            end
        end
        flagControl:ClearAnchors()
        flagControl:SetAnchor(LEFT, EsoKRUI, LEFT, 14 +count*34, 0)
        count = count + 1
    end
    EsoKRUI:SetDimensions(25 +count*34, 50)
    EsoKRUI:SetMouseEnabled(true)
end

-- 插件初始化时调用
function Initialize()
   -- if (addOnName):find("^ZO_") then return end

   -- local defaultVars = { lang = "kr" }
    EsoKR.Settings = ZO_SavedVars:NewAccountWide("EsoKR_Variables", 1, nil, EsoKR.Defaults)
    
   -- if EsoKR.SettingsCharacter.firstInit == true then
	--	SetSetting(SETTING_TYPE_SUBTITLES, SUBTITLE_SETTING_ENABLED, "true")
	--	EsoKR.SettingsCharacter.firstInit = false
	-- end
	
	if EsoKR.Settings.FirstInit == true then
		SetCVar("IgnorePatcherLanguageSetting", "1")
		
		if EsoKR:GetLanguage() == "kr" then
			EsoKR.Settings.FirstInit = false
			EsoKR:ShowMsgBox("欢迎使用微攻略人工版汉化插件", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al插件应用成功！现在将自动应用上次设置的语言。你可以从右下角的国旗更改语言。\n\n\n\n|ac|t256:128:EsoKR/Textures/Donate.dds|t\n\n\n\n|al汉化插件需要你的捐助支持！\n\n由于现在翻译工具的联网功能，服务器的费用是较大的一项开支。所以才公开求打赏捐助。希望各位玩家可以解囊以分担服务器的费用。", 2)
		else
			EsoKR:ShowMsgBox("欢迎使用微攻略人工版汉化插件", "\n\n|ac|t256:128:EsoKR/Textures/EsozhLogo.dds|t\n\n\n|al感谢你安装《上古卷轴 Online》微攻略人工版汉化插件！\n\n看起来你这是首次启动汉化插件，因此要将语言切换到中文。\n\n现在是否要切换语言？", 1)
		end
	end

   -- SetSCTKeyboardFont(Font_Bold .. "|" .. 32 .. "|",FONT_STYLE_OUTLINE_THICK)

    
    -- chinese gamepad hack
    for k, v in pairs( ZO_TOOLTIP_STYLES ) do
        v.fontFace = "EsoKR/fonts/univers57.otf"
    end
    
    for k, v in pairs( ZO_CRAFTING_TOOLTIP_STYLES ) do
        v.fontFace = "EsoKR/fonts/univers57.otf"
    end
    
    for _, flagCode in pairs(EsoKR.Flags) do
        ZO_CreateStringId("SI_BINDING_NAME_"..string.upper(flagCode), string.upper(flagCode))
    end

    EsoKR.RefreshUI()
    
	if not LibAddonMenu2 then return end
	LibAddonMenu2:RegisterAddonPanel(EsoKR.name.."Options", panelData)
	LibAddonMenu2:RegisterOptionControls(EsoKR.name.."Options", optionsTable)

	if EsoKR:GetLanguage() == "kr" then
		EsoKR_init()
	end

    function ZO_GameMenu_OnShow(control)
        if control.OnShow then
            control.OnShow(control.gameMenu)
            EsoKRUI:SetHidden(hidden)
        end
    end
    
    function ZO_GameMenu_OnHide(control)
        if control.OnHide then
            control.OnHide(control.gameMenu)
            EsoKRUI:SetHidden(not hidden)
        end
    end

    ZO_PreHook("ZO_ChatTextEntry_Execute", function(control) control.system:CloseTextEntry(true) end)
    ZO_PreHook("ZO_ChatTextEntry_Escape", function(control) control.system:CloseTextEntry(true) end)
    ZO_PreHook("ZO_ChatTextEntry_TextChanged", function(control, newText) EsoKR:Convert(control.system.textEntry) end)
    ZO_PreHook("ZO_EditDefaultText_OnTextChanged", function(edit) EsoKR:Convert(edit) end)
	
end

function EsoKR:CloseMsgBox()
	ZO_Dialogs_ReleaseDialog("EsoKRDialog", false)
end


function EsoKR_init()

  LMP.MediaTable.font["Univers 67"] = nil 
  LMP.MediaTable.font["Univers 57"] = nil
  LMP.MediaTable.font["Univers 55"] = nil
  LMP.MediaTable.font["Skyrim Handwritten"] = nil
  LMP.MediaTable.font["ProseAntique"] = nil
  LMP.MediaTable.font["Trajan Pro"] = nil
  LMP:Register("font", "Univers 67",  Font_Bold)
  LMP:Register("font", "Univers 57", Font_Medium)
  LMP:Register("font", "Univers 55", Font_Light)
  LMP:Register("font", "Skyrim Handwritten", "EsoKR/fonts/handwritten_bold.otf")
  LMP:Register("font", "ProseAntique", "EsoKR/fonts/proseantiquepsmt.otf")
  LMP:Register("font", "Trajan Pro", "EsoKR/fonts/trajanpro-regular.otf")
  LMP:SetDefault("font", Font_Medium)
	
	if LWF3 then
		LWF3.data.Fonts = {
			["ESO Cartographer"] = Font_Medium,
			["Fontin Bold"] = Font_Medium,
			["Fontin Italic"] = Font_Medium,
			["Fontin Regular"] = Font_Medium,
			["Fontin SmallCaps"] = Font_Medium,
			["ProseAntique"] = "EsoKR/fonts/proseantiquepsmt.otf",
			["Skyrim Handwritten"]= "EsoKR/fonts/handwritten_bold.otf",
			["Trajan Pro"] = "EsoKR/fonts/trajanpro-regular.otf",
			["Univers 55"] = Font_Light,
			["Univers 57"] = Font_Medium,
			["Univers 67"] = Font_Bold,
		}
	end
	
	if LWF4 then
		LWF4.data.Fonts = {
			["ESO Cartographer"] = Font_Medium,
			["Fontin Bold"] = Font_Bold,
			["Fontin Italic"] = Font_Medium,
			["Fontin Regular"] = Font_Medium,
			["Fontin SmallCaps"] = Font_Medium,
			["ProseAntique"] = "EsoKR/fonts/proseantiquepsmt.otf",
			["Skyrim Handwritten"]= "EsoKR/fonts/handwritten_bold.otf",
			["Trajan Pro"] = "EsoKR/fonts/trajanpro-regular.otf",
			["Univers 55"] = Font_Light,
			["Univers 57"] = Font_Medium,
			["Univers 67"] = Font_Bold,
		}
	end

end

function EsoKR:ShowMsgBox(title, msg, typ)

	local callback = {}
	
	if typ == 1 then
		callback = {
			[1] = 
			{
				keybind = "DIALOG_PRIMARY",
				text = "确定切换", 
				callback =
					function ()
						EsoKR.ChangeLang("kr")
					end,
                clickSound = SOUNDS.DIALOG_ACCEPT,
			},
			[2] =
			{
				keybind = "DIALOG_NEGATIVE",
                text = "取消",
                clickSound = SOUNDS.DIALOG_DECLINE,
			},
		}
	elseif typ == 2 then
		callback = {
			[1] = 
			{
				keybind = "DIALOG_PRIMARY",
				text = SI_OK,
                clickSound = SOUNDS.DIALOG_ACCEPT,
			}
		}
	elseif typ == 3 then
		callback = {
			[1] = 
			{
				keybind = "DIALOG_PRIMARY",
				text = "访问网站", 
				callback =
					function ()
						RequestOpenUnsafeURL("https://vstab.com/local/")
					end,
                clickSound = SOUNDS.DIALOG_ACCEPT,
			},
			[2] =
			{
				keybind = "DIALOG_NEGATIVE",
                text = "取消",
                clickSound = SOUNDS.DIALOG_DECLINE,
			},
		}
		elseif typ == 4 then
		callback = {
			[1] = 
			{
				keybind = "DIALOG_PRIMARY",
				text = "访问Steam社区组", 
				callback =
					function ()
						RequestOpenUnsafeURL("https://steamcommunity.com/groups/ESOChina")
					end,
                clickSound = SOUNDS.DIALOG_ACCEPT,
			},
			[2] =
			{
				keybind = "DIALOG_NEGATIVE",
                text = "取消",
                clickSound = SOUNDS.DIALOG_DECLINE,
			},
		}
	elseif typ == 5 then
		callback = {
			[1] = 
			{
				keybind = "DIALOG_PRIMARY",
				text = "访问汉化组招新申请页面", 
				callback =
					function ()
						RequestOpenUnsafeURL("https://www.edigame.com/join")
					end,
                clickSound = SOUNDS.DIALOG_ACCEPT,
			},
			[2] =
			{
				keybind = "DIALOG_NEGATIVE",
                text = "取消",
                clickSound = SOUNDS.DIALOG_DECLINE,
			},
		}
	end
	
	local confirmDialog = 
	{
		canQueue = true,
		onlyQueueOnce = true,
		gamepadInfo = { dialogType = GAMEPAD_DIALOGS.BASIC },
		title = { text = title },
		mainText = { text = msg },
		buttons = callback
	}
	
	ZO_Dialogs_RegisterCustomDialog("EsoKRDialog", confirmDialog)
	EsoKR:CloseMsgBox()
	
	--else
		ZO_Dialogs_ShowDialog("EsoKRDialog")
	--end
end

-- 玩家激活时调用
function EsoKR.LoadScreen(event)
    SetNameplateKeyboardFont(Font_Bold .. "|" .. 22 .. "|",FONT_STYLE_OUTLINE_THICK)
    SetNameplateGamepadFont(Font_Bold .. "|" .. 22 .. "|",FONT_STYLE_OUTLINE_THICK)
    
    -- 检查当前语言、上次设置的语言是否是中文，如有必要就修改语言设置
   -- if EsoKR.firstInit then
    --    EsoKR.firstInit = false
     --   if EsoKR.GetLanguage() ~= "kr" and EsoKR.savedVars.lang == "kr" then
      --      EsoKR.ChangeLang("kr")
      --  end
  --  end
end

-- 修改字幕显示时间，因为中文文本一般会比英文短
if not EsoKR.old_OnShowSubtitle then
    EsoKR.old_OnShowSubtitle = ZO_SubtitleManager.OnShowSubtitle
end
ZO_SubtitleManager.OnShowSubtitle = function(self, ...)
    EsoKR.old_OnShowSubtitle(self, ...)
    local message = self.currentSubtitle and self.currentSubtitle.messageText
    local lang = GetCVar("Language.2")
    if lang == "kr" and message and EsoKR.IsAscii(message) then
        local minLen = 3
        local maxLen = 12
        local messageLength = ZoUTF8StringLength(message)
        local charactersPerSecond = 4  -- default is 10
        self.currentSubtitle.displayLengthSeconds = zo_clamp(messageLength / charactersPerSecond, minLen, maxLen)
    end
end

-- 注册初始化事件

function OnAddonLoaded(eventType, addonName)
	if addonName == EsoKR.name then
		Initialize()
		--UnregisterForEvent--
		EVENT_MANAGER:UnregisterForEvent(EsoKR.name, eventType)
	end
end
EVENT_MANAGER:RegisterForEvent(EsoKR.name, EVENT_ADD_ON_LOADED, OnAddonLoaded)
--EVENT_MANAGER:RegisterForEvent("EsoKR_OnAddOnLoaded", EVENT_ADD_ON_LOADED, function(_event, _name) EsoKR.OnInit(_event, _name) end)
EVENT_MANAGER:RegisterForEvent("EsoKR_SCTFix", EVENT_PLAYER_ACTIVATED, function(...) EsoKR:SCTFix() end)
EVENT_MANAGER:RegisterForEvent("EsoKR_LoadScreen", EVENT_PLAYER_ACTIVATED, EsoKR.LoadScreen)
EVENT_MANAGER:RegisterForEvent("EsoKR_StartupMessage", EVENT_PLAYER_ACTIVATED, function(...) EsoKR:StartupMessage() end)
